@@ -18,14 +18,13 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ***************************************************************************/
 #include "dht.h"
-#include <qmap.h>
-#include <k3resolver.h>
+#include <QMap>
+#include <net/addressresolver.h>
 #include <util/log.h>
 #include <util/array.h>
 #include <util/error.h>
 #include <util/functions.h>
 #include <bcodec/bnode.h>
-#include <k3socketaddress.h>
 #include "announcetask.h"
 #include "node.h"
 #include "rpcserver.h"
@@ -34,19 +33,26 @@
 #include "database.h"
 #include "taskmanager.h"
 #include "nodelookup.h"
-
+#include "pingreq.h"
+#include "findnodereq.h"
+#include "getpeersreq.h"
+#include "announcereq.h"
+#include "pingrsp.h"
+#include "findnodersp.h"
+#include "getpeersrsp.h"
+#include "announcersp.h"
 
 using namespace bt;
-using namespace KNetwork;
 
 namespace dht
 {
-	
 
 
-	DHT::DHT() : node(0),srv(0),db(0),tman(0),our_node_lookup(0)
+
+	DHT::DHT() : node(0), srv(0), db(0), tman(0), our_node_lookup(0)
 	{
-		connect(&update_timer,SIGNAL(timeout()),this,SLOT(update()));
+		connect(&update_timer, SIGNAL(timeout()), this, SLOT(update()));
+		connect(&expire_timer, SIGNAL(timeout()), this, SLOT(expireDatabaseItems()));
 	}
 
 
@@ -55,27 +61,27 @@ namespace dht
 		if (running)
 			stop();
 	}
-	
-	void DHT::start(const QString & table,const QString & key_file,bt::Uint16 port)
+
+	void DHT::start(const QString & table, const QString & key_file, bt::Uint16 port)
 	{
 		if (running)
 			return;
-		
+
 		if (port == 0)
-			port = 6881; 
-		
+			port = 6881;
+
 		table_file = table;
 		this->port = port;
-		Out(SYS_DHT|LOG_NOTICE) << "DHT: Starting on port " << port << endl;
-		srv = new RPCServer(this,port);
-		node = new Node(srv,key_file);
+		Out(SYS_DHT | LOG_NOTICE) << "DHT: Starting on port " << port << endl;
+		srv = new RPCServer(this, port);
+		node = new Node(srv, key_file);
 		db = new Database();
-		tman = new TaskManager();
-		expire_timer.update();
+		tman = new TaskManager(this);
 		running = true;
 		srv->start();
 		node->loadTable(table);
 		update_timer.start(1000);
+		expire_timer.start(5*60*1000);
 		started();
 		if (node->getNumEntriesInRoutingTable() > 0)
 		{
@@ -83,15 +89,16 @@ namespace dht
 			findOwnNode();
 		}
 	}
-		
-		
+
+
 	void DHT::stop()
 	{
 		if (!running)
 			return;
-		
+
 		update_timer.stop();
-		Out(SYS_DHT|LOG_NOTICE) << "DHT: Stopping " << endl;
+		expire_timer.stop();
+		Out(SYS_DHT | LOG_NOTICE) << "DHT: Stopping " << endl;
 		srv->stop();
 		node->saveTable(table_file);
 		running = false;
@@ -102,44 +109,46 @@ namespace dht
 		delete srv; srv = 0;
 	}
 
-	void DHT::ping(PingReq::Ptr r)
+	void DHT::ping(const PingReq & r)
 	{
 		if (!running)
 			return;
-		
+
 		// ignore requests we get from ourself
-		if (r->getID() == node->getOurID())
+		if (r.getID() == node->getOurID())
 			return;
-		
-		Out(SYS_DHT|LOG_NOTICE) << "DHT: Sending ping response" << endl;
-		PingRsp rsp(r->getMTID(),node->getOurID());
-		rsp.setOrigin(r->getOrigin());
+
+		PingRsp rsp(r.getMTID(), node->getOurID());
+		rsp.setOrigin(r.getOrigin());
 		srv->sendMsg(rsp);
-		node->received(this,r);
+		node->received(this, r);
 	}
-	
-	
-	
-	void DHT::findNode(FindNodeReq::Ptr r)
+
+	void DHT::findNode(const dht::FindNodeReq& r)
 	{
 		if (!running)
 			return;
-		
+
 		// ignore requests we get from ourself
-		if (r->getID() == node->getOurID())
+		if (r.getID() == node->getOurID())
 			return;
-		
-		Out(SYS_DHT|LOG_DEBUG) << "DHT: got findNode request" << endl;
-		node->received(this,r);
+
+		node->received(this, r);
 		// find the K closest nodes and pack them
-		KClosestNodesSearch kns(r->getTarget(),K);
-		
-		node->findKClosestNodes(kns);
-	
-		FindNodeRsp fnr(r->getMTID(),node->getOurID());
+		KClosestNodesSearch kns(r.getTarget(), K);
+
+		bt::Uint32 wants = 0;
+		if (r.wants(4) || r.getOrigin().ipVersion() == 4)
+			wants |= WANT_IPV4;
+		if (r.wants(6) || r.getOrigin().ipVersion() == 6)
+			wants |= WANT_IPV6;
+
+		node->findKClosestNodes(kns, wants);
+
+		FindNodeRsp fnr(r.getMTID(), node->getOurID());
 		// pack the found nodes in a byte array
 		kns.pack(&fnr);
-		fnr.setOrigin(r->getOrigin());
+		fnr.setOrigin(r.getOrigin());
 		srv->sendMsg(fnr);
 	}
 
@@ -147,10 +156,10 @@ namespace dht
 	{
 		if (our_node_lookup)
 			return our_node_lookup;
-		
+
 		our_node_lookup = findNode(node->getOurID());
 		if (our_node_lookup)
-			connect(our_node_lookup,SIGNAL(finished(Task*)),this,SLOT(ownNodeLookupFinished(Task*)));
+			connect(our_node_lookup, SIGNAL(finished(Task*)), this, SLOT(ownNodeLookupFinished(Task*)));
 		return our_node_lookup;
 	}
 
@@ -160,95 +169,89 @@ namespace dht
 			our_node_lookup = 0;
 	}
 
-	
-	void DHT::announce(AnnounceReq::Ptr r)
+
+	void DHT::announce(const AnnounceReq & r)
 	{
 		if (!running)
 			return;
-		
+
 		// ignore requests we get from ourself
-		if (r->getID() == node->getOurID())
+		if (r.getID() == node->getOurID())
 			return;
-		
-		Out(SYS_DHT|LOG_DEBUG) << "DHT: got announce request" << endl;
-		node->received(this,r);
+
+		node->received(this, r);
 		// first check if the token is OK
-		dht::Key token = r->getToken();
-		if (!db->checkToken(token,r->getOrigin()))
+		dht::Key token = r.getToken();
+		if (!db->checkToken(token, r.getOrigin()))
 			return;
-		
+
 		// everything OK, so store the value
-		db->store(r->getInfoHash(),DBItem(r->getOrigin()));
+		db->store(r.getInfoHash(), DBItem(r.getOrigin()));
 		// send a proper response to indicate everything is OK
-		AnnounceRsp rsp(r->getMTID(),node->getOurID());
-		rsp.setOrigin(r->getOrigin());
+		AnnounceRsp rsp(r.getMTID(), node->getOurID());
+		rsp.setOrigin(r.getOrigin());
 		srv->sendMsg(rsp);
 	}
-	
-	
-	
-	void DHT::getPeers(GetPeersReq::Ptr r)
-	{
-		if (!running)
-			return;
-		
-		// ignore requests we get from ourself
-		if (r->getID() == node->getOurID())
-			return;
-		
-		Out(SYS_DHT|LOG_DEBUG) << "DHT: got getPeers request" << endl;
-		node->received(this,r);
-		DBItemList dbl;
-		db->sample(r->getInfoHash(),dbl,50);
-		
-		// generate a token
-		dht::Key token = db->genToken(r->getOrigin());
-		
-		if (dbl.count() == 0)
-		{
-			// if data is null do the same as when we have a findNode request
-			// find the K closest nodes and pack them
-			KClosestNodesSearch kns(r->getInfoHash(),K);
-			node->findKClosestNodes(kns);
-			
-		
-			GetPeersRsp fnr(r->getMTID(),node->getOurID(),token);
-			kns.pack(&fnr);
-			fnr.setOrigin(r->getOrigin());
-			srv->sendMsg(fnr);
-		}
-		else
-		{			
-			// send a get peers response
-			GetPeersRsp fvr(r->getMTID(),node->getOurID(),dbl,token);
-			fvr.setOrigin(r->getOrigin());
-			srv->sendMsg(fvr);
-		}
-	}
-	
-	void DHT::response(MsgBase::Ptr r)
-	{
-		if (!running)
-			return;
-		
-		node->received(this,r);
-	}
-	
-	void DHT::error(ErrMsg::Ptr )
-	{}
-	
 
-	void DHT::portReceived(const QString & ip,bt::Uint16 port)
+
+
+	void DHT::getPeers(const GetPeersReq & r)
 	{
 		if (!running)
 			return;
-		
-		Out(SYS_DHT|LOG_DEBUG) << "Sending ping request to " << ip << ":" << port << endl;
-		MsgBase::Ptr r(new PingReq(node->getOurID()));
-		r->setOrigin(KInetSocketAddress(ip,port));
+
+		// ignore requests we get from ourself
+		if (r.getID() == node->getOurID())
+			return;
+
+		node->received(this, r);
+		DBItemList dbl;
+		db->sample(r.getInfoHash(), dbl, 50, r.getOrigin().ipVersion());
+
+		// generate a token
+		dht::Key token = db->genToken(r.getOrigin());
+
+		bt::Uint32 wants = 0;
+		if (r.wants(4) || r.getOrigin().ipVersion() == 4)
+			wants |= WANT_IPV4;
+		if (r.wants(6) || r.getOrigin().ipVersion() == 6)
+			wants |= WANT_IPV6;
+
+		// if data is null do the same as when we have a findNode request
+		// find the K closest nodes and pack them
+		KClosestNodesSearch kns(r.getInfoHash(), K);
+		node->findKClosestNodes(kns, wants);
+
+		GetPeersRsp fnr(r.getMTID(), node->getOurID(), dbl, token);
+		kns.pack(&fnr);
+		fnr.setOrigin(r.getOrigin());
+		srv->sendMsg(fnr);
+	}
+
+	void DHT::response(const RPCMsg & r)
+	{
+		if (!running)
+			return;
+
+		node->received(this, r);
+	}
+
+	void DHT::error(const ErrMsg & msg)
+	{
+		Q_UNUSED(msg);
+	}
+
+
+	void DHT::portReceived(const QString & ip, bt::Uint16 port)
+	{
+		if (!running)
+			return;
+
+		RPCMsg::Ptr r(new PingReq(node->getOurID()));
+		r->setOrigin(net::Address(ip, port));
 		srv->doCall(r);
 	}
-	
+
 	bool DHT::canStartTask() const
 	{
 		// we can start a task if we have less then  7 runnning and
@@ -257,116 +260,121 @@ namespace dht
 			return false;
 		else if (256 - srv->getNumActiveRPCCalls() <= 16)
 			return false;
-		
-		return true;	
+
+		return true;
 	}
-	
-	AnnounceTask* DHT::announce(const bt::SHA1Hash & info_hash,bt::Uint16 port)
+
+	AnnounceTask* DHT::announce(const bt::SHA1Hash & info_hash, bt::Uint16 port)
 	{
 		if (!running)
 			return 0;
-		
-		KClosestNodesSearch kns(info_hash,K);
-		node->findKClosestNodes(kns);
+
+		KClosestNodesSearch kns(info_hash, K);
+		node->findKClosestNodes(kns, WANT_BOTH);
 		if (kns.getNumEntries() > 0)
 		{
-			Out(SYS_DHT|LOG_NOTICE) << "DHT: Doing announce " << endl;
-			AnnounceTask* at = new AnnounceTask(db,srv,node,info_hash,port);
-			at->start(kns,!canStartTask());
+			Out(SYS_DHT | LOG_NOTICE) << "DHT: Doing announce " << endl;
+			AnnounceTask* at = new AnnounceTask(db, srv, node, info_hash, port, tman);
+			at->start(kns, !canStartTask());
 			tman->addTask(at);
 			if (!db->contains(info_hash))
 				db->insert(info_hash);
 			return at;
 		}
-		
+
 		return 0;
 	}
-	
-	NodeLookup* DHT::refreshBucket(const dht::Key & id,KBucket & bucket)
+
+	NodeLookup* DHT::refreshBucket(const dht::Key & id, KBucket & bucket)
 	{
 		if (!running)
 			return 0;
-		
-		KClosestNodesSearch kns(id,K);
+
+		KClosestNodesSearch kns(id, K);
 		bucket.findKClosestNodes(kns);
 		bucket.updateRefreshTimer();
 		if (kns.getNumEntries() > 0)
 		{
-			Out(SYS_DHT|LOG_DEBUG) << "DHT: refreshing bucket " << endl;
-			NodeLookup* nl = new NodeLookup(id,srv,node);
-			nl->start(kns,!canStartTask());
+			Out(SYS_DHT | LOG_DEBUG) << "DHT: refreshing bucket " << endl;
+			NodeLookup* nl = new NodeLookup(id, srv, node, tman);
+			nl->start(kns, !canStartTask());
 			tman->addTask(nl);
 			return nl;
 		}
-		
+
 		return 0;
 	}
-	
+
 	NodeLookup* DHT::findNode(const dht::Key & id)
 	{
 		if (!running)
 			return 0;
-		
-		KClosestNodesSearch kns(id,K);
-		node->findKClosestNodes(kns);
+
+		KClosestNodesSearch kns(id, K);
+		node->findKClosestNodes(kns, WANT_BOTH);
 		if (kns.getNumEntries() > 0)
 		{
-			Out(SYS_DHT|LOG_DEBUG) << "DHT: finding node " << endl;
-			NodeLookup* at = new NodeLookup(id,srv,node);
-			at->start(kns,!canStartTask());
+			Out(SYS_DHT | LOG_DEBUG) << "DHT: finding node " << endl;
+			NodeLookup* at = new NodeLookup(id, srv, node, tman);
+			at->start(kns, !canStartTask());
 			tman->addTask(at);
 			return at;
 		}
-		
+
 		return 0;
 	}
-	
+
+	void DHT::expireDatabaseItems()
+	{
+		db->expire(bt::CurrentTime());
+	}
+
+
 	void DHT::update()
 	{
 		if (!running)
 			return;
-		
+
 		try
 		{
-			if (expire_timer.getElapsedSinceUpdate() > 5*60*1000)
-			{
-				db->expire(bt::CurrentTime());
-				expire_timer.update();
-			}
-			
-			srv->handlePackets();
 			node->refreshBuckets(this);
-			tman->removeFinishedTasks(this);
 			stats.num_tasks = tman->getNumTasks() + tman->getNumQueuedTasks();
 			stats.num_peers = node->getNumEntriesInRoutingTable();
 		}
 		catch (bt::Error & e)
 		{
-			Out(SYS_DHT|LOG_IMPORTANT) << "DHT: Error: " << e.toString() << endl;
+			Out(SYS_DHT | LOG_IMPORTANT) << "DHT: Error: " << e.toString() << endl;
 		}
 	}
-	
-	void DHT::timeout(MsgBase::Ptr r)
+
+	void DHT::timeout(RPCMsg::Ptr r)
 	{
 		node->onTimeout(r);
 	}
-	
-	void DHT::addDHTNode(const QString & host,Uint16 hport)
+
+	void DHT::addDHTNode(const QString & host, Uint16 hport)
 	{
 		if (!running)
 			return;
-		
-		KResolver::resolveAsync(this,SLOT(onResolverResults(KNetwork::KResolverResults)),host,QString::number(hport));
-	}
-	
-	void DHT::onResolverResults(KNetwork::KResolverResults res)
-	{
-		if (!running)
-			return;
-		
-		if (res.count() > 0)
+
+		net::Address addr;
+		if (addr.setAddress(host))
 		{
-			srv->ping(node->getOurID(),net::Address(res.front().address()));
+			addr.setPort(hport);
+			srv->ping(node->getOurID(), addr);
+		}
+		else
+			net::AddressResolver::resolve(host, hport, this, SLOT(hostResolved(net::AddressResolver*)));
+	}
+
+	void DHT::onResolverResults(net::AddressResolver* res)
+	{
+		if (!running)
+			return;
+
+		if (res->succeeded())
+		{
+			srv->ping(node->getOurID(), res->address());
 		}
 	}
 	
@@ -374,30 +382,29 @@ namespace dht
 	{
 		QMap<QString, int> map;
 		
-		if(!node)
+		if (!node)
 			return map;
 		
 		int max = 0;
 		KClosestNodesSearch kns(node->getOurID(), maxNodes*2);
-		node->findKClosestNodes(kns);
+		node->findKClosestNodes(kns, WANT_BOTH);
 		
 		KClosestNodesSearch::Itr it;
-		for(it = kns.begin(); it != kns.end(); ++it)
+		for (it = kns.begin(); it != kns.end(); ++it)
 		{
 			KBucketEntry e = it->second;
 			
-			if(!e.isGood())
+			if (!e.isGood())
 				continue;
 			
-			KInetSocketAddress a = e.getAddress();
+			const net::Address & a = e.getAddress();
 			
-			map.insert(a.ipAddress().toString(), a.port());
-			if(++max >= maxNodes)
+			map.insert(a.toString(), a.port());
+			if (++max >= maxNodes)
 				break;
 		}
 		
 		return map;
 	}
+	
 }
-
-#include "dht.moc"
