@@ -38,9 +38,6 @@
 
 namespace bt
 {
-	
-	Uint32 ChunkManager::max_chunk_size_for_data_check = 512 * 1024;
-	bool ChunkManager::do_data_check = true;
 	Uint32 ChunkManager::preview_size_audio = 256 * 1024; // 256 KB for audio files
 	Uint32 ChunkManager::preview_size_video = 2048 * 1024; // 2 MB for videos
 	
@@ -80,8 +77,6 @@ namespace bt
 		BitSet todo;
 		mutable Uint32 chunks_left;
 		mutable bool recalc_chunks_left;
-		Uint32 corrupted_count;
-		Uint32 recheck_counter;
 		bool during_load;	
 		QSet<Uint32> border_chunks;
 	};
@@ -175,6 +170,12 @@ namespace bt
 	{
 		return d->cache->hasMissingFiles(sl);
 	}
+	
+	bool ChunkManager::isStorageMounted(QStringList & missing)
+	{
+		return d->cache->isStorageMounted(missing);
+	}
+
 
 	Chunk* ChunkManager::getChunk(unsigned int i)
 	{
@@ -608,15 +609,15 @@ namespace bt
 		return d->cache->getOutputPath();
 	}
 	
-	void ChunkManager::preallocateDiskSpace(PreallocationThread* prealloc)
+	void ChunkManager::preparePreallocation(PreallocationThread* prealloc)
 	{
-		d->cache->preallocateDiskSpace(prealloc);
+		d->cache->preparePreallocation(prealloc);
 	}
 	
-	void ChunkManager::dataChecked(const BitSet & ok_chunks)
+	void ChunkManager::dataChecked(const bt::BitSet& ok_chunks, bt::Uint32 from, bt::Uint32 to)
 	{
 		// go over all chunks at check each of them
-		for (Uint32 i = 0;i < (Uint32)d->chunks.size();i++)
+		for (Uint32 i = from;i < (Uint32)d->chunks.size() && i <= to;i++)
 		{
 			Chunk* c = d->chunks[i];
 			if (ok_chunks.get(i) && !bitset.get(i))
@@ -659,7 +660,6 @@ namespace bt
 			Out(SYS_DIO|LOG_DEBUG) << "Failed to save index file : unknown exception" << endl;
 		}
 		chunksLeft();
-		d->corrupted_count = 0;
 	}
 	
 	bool ChunkManager::hasExistingFiles() const
@@ -738,7 +738,6 @@ namespace bt
 			Out(SYS_DIO|LOG_DEBUG) << "Failed to save index file : unknown exception" << endl;
 		}
 		chunksLeft();
-		d->corrupted_count = 0;
 	}
 	
 	void ChunkManager::recreateMissingFiles()
@@ -848,6 +847,7 @@ namespace bt
 	void ChunkManager::loadIndexFile()
 	{
 		d->loadIndexFile();
+		d->cache->loadMountPoints();
 	}
 
 	
@@ -875,6 +875,7 @@ namespace bt
 			cache = fac->create(tor,tmpdir,datadir);
 		
 		cache->loadFileMap();
+		
 		index_file = tmpdir + "index";
 		file_info_file = tmpdir + "file_info";
 		file_priority_file = tmpdir + "file_priority";
@@ -892,7 +893,6 @@ namespace bt
 		
 		chunks_left = 0;
 		recalc_chunks_left = true;
-		corrupted_count = recheck_counter = 0;
 	}
 	
 	ChunkManager::Private::~Private()
