@@ -116,10 +116,7 @@ namespace bt
 	void MagnetDownloader::update()
 	{
 		if (pman)
-		{
-			pman->connectToPeers();
 			pman->update();
-		}
 	}
 	
 	bool MagnetDownloader::running() const
@@ -160,6 +157,12 @@ namespace bt
 	{
 		return 0;
 	}
+	
+	bool MagnetDownloader::isPartialSeed() const
+	{
+		return false;
+	}
+
 
 	const bt::SHA1Hash& MagnetDownloader::infoHash() const
 	{
